/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * CliqueBlobSystem
 * A particle system which treats groups of particles as 'spring cliques',
 * in which each particle is connected to all others via springs.  Also
 * includes forces for air friction, room boundary collision and random jiggle.
 */

#ifndef __CLIQUE_BLOBSYSTEM_HPP__
#define __CLIQUE_BLOBSYSTEM_HPP__

#include <cstdlib>
#include <vector>
#include "ParticleSystem.hpp"

using std::vector;

class CliqueBlobSystem : public ParticleSystem{
public:
	//Creates a new empty CliqueBlobSystem
	CliqueBlobSystem() : parts(){}
	//Deletes all particles contained in the system
	virtual ~CliqueBlobSystem();

	//Add the given particle to the system and take ownership of it.
	virtual void addParticle(Particle& p);

	//Returns a vector containing pointers to all particles owned by the
	//system.
	virtual vector<Particle*> getParticles() const;

	//Sets the size of the room for the room collision force
	void setRoomBounds(double xMax, double yMax, double zMax);

	//Adds this system's appropriate forces to the particles.
	virtual void updateForces();

	//Applies the accumulated forces for each particle, changing their
	//position appropriately.
	virtual void applyForces();

protected:
	vector<Particle*> parts;
	double xMax, yMax, zMax;

	//Helper functions for applying various types of forces
	void applySprings(Particle*);
	void applyFriction(Particle*);
	void applyJiggle(Particle*);
	void applyRoomCollision(Particle*);
};

#endif
