/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * HandTracker (outdated and unused)
 * Monitors and stores location of the current user's hands, and maps the
 * resulting positions from user-space to program-space.
 */

#ifndef __HAND_TRACKER_HPP__
#define __HAND_TRACKER_HPP__

#include "TripleD.hpp"
#include "PointMapper.hpp"
#include "OscSkeletonListener.hpp"

class HandTracker{
public:
	//Creates a new HandTracker listening to the given port for OSCeleton
	//messages.
	HandTracker(unsigned listenPort) : osl(listenPort), mapper(true),
		state(NO_CALIB) {}

	//Runs a loop that listens for OSCeleton messages on this tracker's
	//designated port.  Does not return.
	void listen();

	//Returns whether user hand data is currently available.
	bool isTrackingHands() const;

	//Assigns the left and right hand positions to the first and second
	//arguments, respectively, iff isTrackingHands returns true.
	//Returns whether data was assigned.
	bool getHandData(TripleD& left, TripleD& right);

	//Causes the tracker to poll for hand position updates.
	void update();

	//Calibrates the mapping of the user-space bounding box to the program-
	//space bounding box.  Successive iterations will potentially expand the
	//user-space box if the user moves outside of the previous one.
	//Uses the two provided points as the minimum and maximum points of the
	//program-space bounding box to map to.
	void triggerCalibration(const TripleD& minRange,
	                        const TripleD& maxRange);

	//Removes calibration settings for position mapping.
	void resetCalibration();

private:
	OscSkeletonListener osl;
	PointMapper mapper;

	TripleD lastLeft, lastRight;

	enum State{
		NO_CALIB,
		CALIB_A,
		CALIB_B
	} state;

	//Helper function
	//Forces the hand positions to be updated, even if no new data is available
	void forceUpdate();
};

#endif
