/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * MotionSmoother
 * Smooths incoming position values to decrease any inherent noise.
 */

#ifndef __MOTION_SMOOTHER_HPP__
#define __MOTION_SMOOTHER_HPP__

#include "TripleD.hpp"

class MotionSmoother{
public:
	//Creates a new MotionSmoother, storing the given number of frames in
	//history.  If histFrames is equal to 0, a ZeroException object will be
	//thrown.
	MotionSmoother(unsigned histFrames);
	~MotionSmoother();

	//Clears the smoother's history frames
	void clear();

	//Add a position value to the smoother's history
	void pushFrame(TripleD pos);

	//Return whether a smoothed position can yet be returned
	bool dataAvailable() const { return currFrames > 0; }

	//Returns the current smoothed position.
	TripleD getPos() const;
	//Returns the current smoothed velocity.
	TripleD getVel() const;

private:
	const unsigned MAX_FRAMES;
	unsigned currFrames;
	TripleD **hist;
	TripleD vel;
};

class ZeroException{};

#endif
