/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * OrbitalBlobSystem (currently unused)
 * A ParticleSystem that attempts to create 'orbiting' behavior in particle
 * clusters
 */

#ifndef __ORBITAL_BLOBSYSTEM_HPP__
#define __ORBITAL_BLOBSYSTEM_HPP__

#include <cstdlib>
#include <vector>
#include "ParticleSystem.hpp"

class OrbitalBlobSystem : public ParticleSystem{
public:
	//Create an empty new OrbitalBlobSystem
	OrbitalBlobSystem() : parts(){}
	//Destroy all particles in the OrbitalBlobSystem
	virtual ~OrbitalBlobSystem();

	//Add the given particle to the system and take ownership of it.
	virtual void addParticle(Particle& p);
	//Returns a vector containing pointers to all particles owned by this
	//particle system
	virtual std::vector<Particle*> getParticles() const;

	//Sets the expected maximum bounds of a room surface, used for collision
	//handling, where the minimum is zero
	void setRoomBounds(double xMax, double yMax, double zMax);

	//Adds this system's appropriate forces to the particles.
	virtual void updateForces();
	//Applies the accumulated forces for each particle, changing their
	//position appropriately.
	virtual void applyForces();

protected:
	std::vector<Particle*> parts;
	double xMax, yMax, zMax;

	//Helper functions for applying various types of forces
	void applyCentripetal();
	void applySprings();
	void applyFriction(Particle*);
	void applyJiggle(Particle*);
	void applyRoomCollision(Particle*);
};

#endif
