/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * OscMessageSender
 * Sends Open Sound Control messages to a client with data about a given set
 * of particles.
 */

#ifndef __OSC_MESSAGE_SENDER_HPP__
#define __OSC_MESSAGE_SENDER_HPP__

#include <vector>
#include <string>
#include "Particle.hpp"
#include "osc/OscOutboundPacketStream.h"
#include "ip/UdpSocket.h"

#pragma comment(lib, "Ws2_32.lib")
using std::string;
using std::vector;

//The supported patches are enumerated here.
//TODO: Make this generic, and provide systems for enumerating and displaying
//information about them dynamically.
enum SoundSynth{
	SPACE = 0,
	GRANULAR = 1,
	STRETCH = 2,
	STOCK = 3,
	NUM_SYNTHS = 4
};

class OscMessageSender{
public:
	//Creates a new OscMessageSender that sends OSC packets to the given target
	//IP address and port.
	OscMessageSender(const char* sip, unsigned int sport);
	~OscMessageSender();

	//Sends a 'patch initialization' message to the client system with the
	//given number of particles and synth choice
	void sendInit(int numParts, SoundSynth synth);
	//Sends a 'clear' message to the client system
	void sendClear();
	//Sends data concerning the given list of particles to the client, scaling
	//all appropriate data by the given amounts in each axis.
	void sendParticleBatch(vector<Particle*> parts, double xScale,
		                   double yScale, double zScale);

private:
	UdpTransmitSocket sock;
	osc::OutboundPacketStream* stream;
	char* buffer;

	//Helper methods
	//Start building a message with the given route tag
	void startMessage(string s);
	//Send the given triple as a series of three floating-point values
	void sendTripleD(const TripleD& t);
	//Stop building the current message
	void endMessage();
};

#endif
