/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * OscSkeletonListener (outdated, used only for test purposes)
 * Listens for OSCeleton messages, using them to latch onto the first user it
 * sees and providing access to data about that user's limbs.
 */
//TODO: Correct concurrency errors (readers/writer pattern or flat
//monitor mutex)

#ifndef __OSC_SKELETON_LISTENER_HPP__
#define __OSC_SKELETON_LISTENER_HPP__

#include "TripleD.hpp"
#include "osc/OscPacketListener.h"
#include "ip/UdpSocket.h"

class OscSkeletonListener : public osc::OscPacketListener{
public:
	enum State{
		PEND_USER,
		PEND_CALIB,
		DATA_OLD,
		DATA_NEW
	};

	//Creates a new OscSkeletonListener that will listen on the given port
	//for OSCeleton UDP state packets
	OscSkeletonListener(unsigned short listenPort) :
		osc::OscPacketListener(),
		sock(IpEndpointName(IpEndpointName::ANY_ADDRESS, listenPort),
		     this),
		state(PEND_USER) {}

	//Returns the current state of the listener.
	State getState() const { return state; }

	//Returns the current data available on the user being tracked via
	//reference assignment.  Results are valid iff getState returns DATA_OLD or
	//DATA_NEW.
	void getData(TripleD& leftOut, TripleD& rightOut,
	             TripleD& torsoOut);

	//Begins listening on the provided socket
	void listen(){ sock.Run(); }

private:
	UdpListeningReceiveSocket sock;
	State state;

	TripleD leftPos, rightPos, torsoPos;
	unsigned userID;

	//Handles incoming OSCeleton messages.
	virtual void ProcessMessage(const osc::ReceivedMessage& m,
	                            const IpEndpointName& remoteEndpoint);

	//Individual event handlers for each handled message type.
	void newUserEvent(const osc::ReceivedMessage& m);
	void newSkelEvent(const osc::ReceivedMessage& m);
	void jointEvent(const osc::ReceivedMessage& m);
	void loseUserEvent(const osc::ReceivedMessage& m);
};

#endif
