/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * ParticleSystem
 * An abstract class for managing a set of particles.  Designed for inheritance
 * where each subclass could handle a specific set of physics rules.
 */

#ifndef __PARTICLESYSTEM_HPP__
#define __PARTICLESYSTEM_HPP__

#include <cstdlib>
#include <vector>
#include "Particle.hpp"

using std::vector;

class ParticleSystem{
public:
	//Creates a new empty particle system
	ParticleSystem() : parts(){}
	//Deletes all particles in the system
	virtual ~ParticleSystem();

	//Add the given particle to the system and take ownership of it.
	virtual void addParticle(Particle& p);
	//Returns a vector containing pointers to all particles owned by this
	//particle system
	virtual vector<Particle*> getParticles() const;

	//Adds this system's appropriate forces to the particles.
	virtual void updateForces() = 0;
	//Applies the accumulated forces for each particle, changing their
	//position appropriately.
	virtual void applyForces();

protected:
	vector<Particle*> parts;
};

#endif
