/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * PointMapper
 * For documentation, see PointMapper.hpp
 */

#include <algorithm>
#include "PointMapper.hpp"

#include <cstdio>

using std::min;
using std::max;

void PointMapper::mapRefA(const TripleD dom, const TripleD rng){
	domA = dom;
	rngA = rng;
	formLines();
	printf("mapRefA set\n");
}

void PointMapper::mapRefB(const TripleD dom, const TripleD rng){
	domB = dom;
	rngB = rng;
	formLines();
	printf("mapRefB set\n");
}

void PointMapper::formLines(){
	convSlope = (rngB - rngA).scale((domB - domA).inverse());
	convInter = rngA - (domA.scale(convSlope));
}

static double clipNum(double rngA, double rngB, double num){
	double minRng = min(rngA, rngB);
	double maxRng = max(rngA, rngB);
	return min(max(minRng, num), maxRng);
}

//Throws 'ZeroDomain' if any of the domain axes have no width.
TripleD PointMapper::mapPoint(const TripleD point) const {
	if(domA.getElemA() == domB.getElemA() ||
	   domA.getElemB() == domB.getElemB() ||
	   domA.getElemC() == domB.getElemC()) throw ZeroDomain();
	TripleD mapped = point.scale(convSlope) + convInter;
	if(!clipToRange)
		return mapped;
	else
		return TripleD(clipNum(rngA.getElemA(),
		                       rngB.getElemA(),
		                       mapped.getElemA()),
		               clipNum(rngA.getElemB(),
		                       rngB.getElemB(),
		                       mapped.getElemB()),
		               clipNum(rngA.getElemC(),
		                       rngB.getElemC(),
		                       mapped.getElemC()));
}

TripleD PointMapper::scalePoint(const TripleD point) const {
	if(domA.getElemA() == domB.getElemA() ||
	   domA.getElemB() == domB.getElemB() ||
	   domA.getElemC() == domB.getElemC()) throw ZeroDomain();
	return point.scale(convSlope);
}
