/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * TripleD
 * A simple triple of floating-point doubles, with useful operations for
 * vector manipulation for use if desired.
 *
 * TODO: Convert to immutable form.  Would require change in all code using it,
 * but would also allow for stronger guarantees and safe pass-by-reference.
 */

#ifndef __TRIPLED_HPP__
#define __TRIPLED_HPP__

class TripleD{
private:
	//Assuming the form (A,B,C)
	double elemA, elemB, elemC;

public:
	//Create a zero-triple
	TripleD() : elemA(0), elemB(0), elemC(0) {}
	//Create a triple containing the given values
	TripleD(double e1, double e2, double e3) : elemA(e1), elemB(e2), elemC(e3){}

	//Accessors for elements A, B and C of the triple
	inline double getElemA() const { return elemA; }
	inline double getElemB() const { return elemB; }
	inline double getElemC() const { return elemC; }

	inline void setElemA(double e){ elemA = e; }
	inline void setElemB(double e){ elemB = e; }
	inline void setElemC(double e){ elemC = e; }

	//Operators - Add, subtract and assign triples component-wise
	//All non-assignment operators return a new copy of the object.
	TripleD operator+(const TripleD &rhs) const;
	TripleD operator-(const TripleD &rhs) const;

	TripleD& operator+=(const TripleD &rhs);
	TripleD& operator-=(const TripleD &rhs);

	TripleD& operator=(const TripleD &rhs);

	//Operator - 'Multiply', here referring to the vector dot product.
	double operator*(const TripleD &rhs) const;

	//Operators - (in)equality
	bool operator==(const TripleD &rhs) const;
	bool operator!=(const TripleD &rhs) const;

	//Returns a new triple with all components scaled by the given value(s)
	TripleD scale(double d) const;
	TripleD scale(double aScale, double bScale, double cScale) const;
	TripleD scale(const TripleD& scales) const;

	//Returns a new triple with each element being the multiplicative
	//inverse of the original.
	TripleD inverse() const;

	//Returns a normalized triple (magnitude 1 if considered a vector)
	TripleD normalize() const;

	//Retrieve the magnitude of the triple.
	double magnitude() const;
};

#endif
