/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * This is the main file of the project.
 * TODO: Serious spring cleaning and reorganization! Too much unorganized stuff
 * in this file that would better serve elsewhere.
 */

#ifdef __APPLE__
#include <GLUT/glut.h>
#include <stdlib.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <GL/freeglut.h>
//#if defined(linux) || defined(__linux)
//#include <stdlib.h>
//#endif
#endif

#include <stdlib.h>
#include <cmath>
#include <time.h>
#include <stdio.h>
#include <iostream>
#include "CliqueBlobSystem.hpp"
#include "OrbitalBlobSystem.hpp"
#include "BlobIsoFunc.hpp"
#include "OscMessageSender.hpp"
#include "InputManager.hpp"

int WIN_WIDTH = 1920;//1280;
int WIN_HEIGHT = 1080;//720;
int WIN_DEPTH = 1000;

#define FULL_SCREEN_STRING "1920x1080:32@60"

#define XCELLS 20
#define YCELLS 20
#define ZCELLS 20

#define NUM_PARTS 12
#ifndef M_2_PI
#define M_2_PI 6.2831853072
#endif
#define SECS_PER_FRAME 0.04

#define SMOOTH_FRAMES 10

void init(void);
void display(void);
void reshape (int w, int h);
void keyboard(unsigned char key, int x, int y);
void update(void);

ParticleSystem* ps;
BlobSurfaceDrawer* blobDraw;
OscMessageSender* oms;
InputManager* im;
SoundSynth synth = SPACE;

int main(int argc, char** argv){
	glutInit(&argc, argv);

	//Select Pixel Format/Device Context Attributes
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutGameModeString(FULL_SCREEN_STRING);
	glutEnterGameMode();

	//Configure Window
	//~ glutInitWindowSize (WIN_WIDTH, WIN_HEIGHT);
	//~ glutInitWindowPosition (30, 10);

	//Create the Window and Set up Rendering Context
	//~ glutCreateWindow ("Kinect Blob");

	//Configure Rendering Context
	init();

	//Connect callback functions that will respond to events
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutIdleFunc(update);

	#pragma omp parallel sections
	{
		#pragma omp section
		{
			im->listen();
		}
		#pragma omp section
		{
			//Start listening for events
			glutMainLoop();
		}
	}

	return 0;
}

void init(void) {
	//Put OpenGL Initializing Code here
	glPointSize(3.0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glPolygonMode(GL_FRONT, GL_FILL);
	glShadeModel(GL_SMOOTH);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glMatrixMode(GL_MODELVIEW);
	glTranslated(WIN_WIDTH - 1, WIN_HEIGHT - 1, WIN_DEPTH - 1);
	float ambLightProps[] = {0.50, 0.50, 0.50, 1.00};
	float difLightProps[] = {0.75, 0.75, 0.75, 1.00};
	float specLightProps[] = {1.00, 1.00, 1.00, 1.00};
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambLightProps);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, difLightProps);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specLightProps);
	//glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0);
	glEnable(GL_LIGHT0);
	glLoadIdentity();
	glTranslated(1, 1, 1);
	glLightfv(GL_LIGHT1, GL_AMBIENT, ambLightProps);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, difLightProps);
	glLightfv(GL_LIGHT1, GL_SPECULAR, specLightProps);
	//glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0);
	glEnable(GL_LIGHT1);
	glLoadIdentity();

	//Other initialization goes here
	srand((unsigned int)time(NULL));

	CliqueBlobSystem* cbs = new CliqueBlobSystem();
	for(int i = 0; i < NUM_PARTS; i++){
		cbs->addParticle(*(new Particle(floor((float)(rand()%(WIN_WIDTH+1))),
									    floor((float)(rand()%(WIN_HEIGHT+1))),
									    floor((float)(rand()%(WIN_DEPTH+1))))));
	}
	cbs->setRoomBounds(WIN_WIDTH, WIN_HEIGHT, WIN_DEPTH);
	ps = cbs;

	blobDraw = new BlobSurfaceDrawer(XCELLS, YCELLS, ZCELLS);
	oms = new OscMessageSender("173.250.173.194", 57120);
	oms->sendInit(NUM_PARTS, synth);
	PointMapper mapper(false);
	mapper.mapRefA(TripleD(0.75, 0.65, 0.5),
	               TripleD(0,0,0));
	mapper.mapRefB(TripleD(0.25, 0.4, 1.5),
	               TripleD(WIN_WIDTH, WIN_HEIGHT, WIN_DEPTH));
	im = new InputManager(ps, 57121, mapper);
}

void drawRoom(){
	glDisable(GL_LIGHTING);
	glBegin(GL_QUADS);
		glColor4f(0.5, 0.5, 0.5, 1.0);
		glNormal3d(-1, 0, 0);
		glVertex3d(0, 0, 0);
		glVertex3d(0, WIN_HEIGHT, 0);
		glVertex3d(0, WIN_HEIGHT, WIN_DEPTH);
		glVertex3d(0, 0, WIN_DEPTH);

		glNormal3d(1, 0, 0);
		glVertex3d(WIN_WIDTH, 0, 0);
		glVertex3d(WIN_WIDTH, 0, WIN_DEPTH);
		glVertex3d(WIN_WIDTH, WIN_HEIGHT, WIN_DEPTH);
		glVertex3d(WIN_WIDTH, WIN_HEIGHT, 0);

		glColor4f(0.4, 0.4, 0.4, 1.0);
		glNormal3d(0, 1, 0);
		glVertex3d(0, WIN_HEIGHT, 0);
		glVertex3d(WIN_WIDTH, WIN_HEIGHT, 0);
		glVertex3d(WIN_WIDTH, WIN_HEIGHT, WIN_DEPTH);
		glVertex3d(0, WIN_HEIGHT, WIN_DEPTH);

		glNormal3d(0, -1, 0);
		glVertex3d(0, 0, 0);
		glVertex3d(0, 0, WIN_DEPTH);
		glVertex3d(WIN_WIDTH, 0, WIN_DEPTH);
		glVertex3d(WIN_WIDTH, 0, 0);

		glColor4f(0.2, 0.2, 0.2, 1.0);
		glNormal3d(0, 0, -1);
		glVertex3d(0, 0, 0);
		glVertex3d(WIN_WIDTH, 0, 0);
		glVertex3d(WIN_WIDTH, WIN_HEIGHT, 0);
		glVertex3d(0, WIN_HEIGHT, 0);
	glEnd();
}

void display(void){

	im->update();

	static TripleD center(WIN_WIDTH/2.0, WIN_HEIGHT/2.0, WIN_DEPTH/2.0);
	vector<Particle*> parts = ps->getParticles();
	for(vector<Particle*>::const_iterator i = parts.begin();
			i != parts.end(); i++)
		(*i)->addForce((center - (*i)->getCurrPos()).normalize().scale(4.0));

	ps->updateForces();

	static double invWidth = 1.0/WIN_WIDTH;
	static double invHeight = 1.0/WIN_HEIGHT;
	static double invDepth = 1.0/WIN_DEPTH;
	oms->sendParticleBatch(ps->getParticles(), invWidth, invHeight, invDepth);

	ps->applyForces();

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClearDepth(1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	drawRoom();
	blobDraw->draw(ps);

	im->drawLimbReps();

	// Required to see drawing if you included GLUT_DOUBLE
	// in your Device Context
	glutSwapBuffers();
}

void reshape (int w, int h){
	if(h == 0)	h = 1;
	WIN_WIDTH = w;
	WIN_HEIGHT = h;
	glViewport(0,0,w,h);
	GLfloat fAspect = (GLfloat)w/(GLfloat)h;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(110.0, fAspect, 0.25, WIN_DEPTH + 400);
	glMatrixMode(GL_MODELVIEW);
	glTranslated(0.0, -WIN_HEIGHT/6.0, -100.0);
	gluLookAt(WIN_WIDTH/2, 3*WIN_HEIGHT/8.0, WIN_DEPTH + 100, WIN_WIDTH/2, WIN_HEIGHT/2, 0.0, 0.0, 1.0, 0.0);
}

void keyboard(unsigned char key, int x, int y){
	switch (key) {
		case 27:
		case 'q':
		case 'Q':
			oms->sendClear();
			delete ps;
			ps = NULL;
			delete blobDraw;
			blobDraw = NULL;
			delete oms;
			oms = NULL;
			delete im;
			im = NULL;
			exit(0);
			break;
		//Controls for manipulating a single point.  Test purposes only.
		case 'l':
		case 'L':
			ps->getParticles()[0]->addForce(TripleD(1000.0, 0.0, 0.0));
			break;
		case 'j':
		case 'J':
			ps->getParticles()[0]->addForce(TripleD(-1000.0, 0.0, 0.0));
			break;
		case 'i':
		case 'I':
			ps->getParticles()[0]->addForce(TripleD(0.0, 1000.0, 0.0));
			break;
		case 'k':
		case 'K':
			ps->getParticles()[0]->addForce(TripleD(0.0, -1000.0, 0.0));
			break;
		case 'u':
		case 'U':
			ps->getParticles()[0]->addForce(TripleD(0.0, 0.0, 1000.0));
			break;
		case 'o':
		case 'O':
			ps->getParticles()[0]->addForce(TripleD(0.0, 0.0, -1000.0));
			break;
		case 's':
		case 'S':
			synth = (SoundSynth)((synth + 1) % NUM_SYNTHS);
			oms->sendInit(NUM_PARTS, synth);
			printf("Synth: %d - %s\n", (int)synth,
				synth == SPACE    ? "Space"    :
				synth == GRANULAR ? "Granular" :
				synth == STRETCH  ? "Stretch"  :
				synth == STOCK    ? "Stock"    :
				                    "Unknown");
			break;
		case 'r':
		case 'R':
			oms->sendInit(NUM_PARTS, synth);
			break;
		case 'c':
		case 'C':
			oms->sendClear();
			break;
		case 'f':
		case 'F':{
			Particle* p = ps->getParticles()[1];
			p->setPhysicsActive(!p->isPhysicsActive());
			break; }
	}
}

void update(void){
	static clock_t lastframe = clock();
	clock_t now = clock();
	if((now - lastframe) >= SECS_PER_FRAME * CLOCKS_PER_SEC){
		glutPostRedisplay();
		lastframe = now;
	}
}
